declare enum ProtocolEnum {
    ARWEAVE = "arweave",
    FILECOIN = "ipfs-filecoin",
    IPFS = "ipfs"
}

interface UploadResult {
    uploadId: string;
    bucketId: string;
    protocolLink: string;
    dynamicLinks: string[];
}

declare enum ProjectStateEnum {
    MAINTAINED = "MAINTAINED",
    ARCHIVED = "ARCHIVED"
}
declare enum DomainTypeEnum {
    DOMAIN = "domain",
    SUBDOMAIN = "subdomain",
    HANDSHAKE_DOMAIN = "handshake-domain",
    HANDSHAKE_SUBDOMAIN = "handshake-subdomain",
    ENS_DOMAIN = "ens-domain"
}

interface TokenScope {
    user: {
        id: string;
        username: string;
        name: string;
        email: string;
    };
    organizations: {
        id: string;
        name: string;
        username: string;
    }[];
}
interface UsageWithLimits {
    usedBandwidth?: number;
    usedBuildExecution?: number;
    usedConcurrentBuild?: number;
    usedStorageArweave?: number;
    usedStorageFileCoin?: number;
    usedStorageIPFS?: number;
    usedDeploymentsPerDay?: number;
    lastDeploymentDate?: Date;
    usedDomains?: number;
    usedHnsDomains?: number;
    usedEnsDomains?: number;
    usedEnvironments?: number;
    usedClusterAkt?: number;
    usedClusterBuildExecution?: number;
    usedNumberOfRequests?: number;
    usedPasswordProtections?: number;
    membersLimit?: number;
    bandwidthLimit?: number;
    buildExecutionLimit?: number;
    concurrentBuildLimit?: number;
    storageArweaveLimit?: number;
    storageFileCoinLimit?: number;
    storageIPFSLimit?: number;
    deploymentsPerDayLimit?: number;
    domainsLimit?: number;
    hnsDomainsLimit?: number;
    ensDomainsLimit?: number;
    environmentsLimit?: number;
    clusterAktLimit?: number;
    clusterBuildExecutionLimit?: number;
    passwordProtectionLimit?: number;
}
interface IPNSName {
    id: string;
    publishedUploadId: string;
    organizationId: string;
    createdAt: string;
    updatedAt: string;
    ipnsHash: string;
    ipnsLink: string;
}

interface Domain {
    id: string;
    name: string;
    link: string;
    verified: boolean;
    bucketId: string;
    type: DomainTypeEnum;
}
type BucketStateEnum = ProjectStateEnum;
interface Bucket {
    id: string;
    name: string;
    organizationId: string;
    state: BucketStateEnum;
    domains: Domain[];
}
declare enum UploadStatusEnum {
    PENDING = "Pending",
    CANCELED = "Canceled",
    DEPLOYED = "Deployed",
    FAILED = "Failed",
    TIMED_OUT = "TimedOut"
}
interface Upload {
    id: string;
    protocolLink: string;
    buildDirectory: string[];
    status: UploadStatusEnum;
    memoryUsed: number;
    bucketId: string;
    protocol: string;
}

declare const ipfs: {
    utils: {
        toV0: (v1: string) => string;
        toV1: (v0: string) => string;
    };
};

interface SpheronClientConfiguration {
    token: string;
}
declare class SpheronClient {
    private readonly configuration;
    private readonly spheronApi;
    private readonly bucketManager;
    private readonly uploadManager;
    constructor(configuration: SpheronClientConfiguration);
    upload(path: string, configuration: {
        name: string;
        protocol: ProtocolEnum;
        organizationId?: string;
        onUploadInitiated?: (uploadId: string) => void;
        onChunkUploaded?: (uploadedSize: number, totalSize: number) => void;
    }): Promise<UploadResult>;
    getBucket(bucketId: string): Promise<Bucket>;
    getBucketDomains(bucketId: string): Promise<Domain[]>;
    getBucketDomain(bucketId: string, domainIdentifier: string): Promise<Domain>;
    addBucketDomain(bucketId: string, options: {
        link: string;
        type: DomainTypeEnum | "domain" | "subdomain" | "handshake-domain" | "handshake-subdomain" | "ens-domain";
        name: string;
    }): Promise<Domain>;
    updateBucketDomain(bucketId: string, domainIdentifier: string, options: {
        link: string;
        name: string;
    }): Promise<Domain>;
    verifyBucketDomain(bucketId: string, domainIdentifier: string): Promise<Domain>;
    deleteBucketDomain(bucketId: string, domainIdentifier: string): Promise<void>;
    archiveBucket(bucketId: string): Promise<void>;
    unarchiveBucket(bucketId: string): Promise<void>;
    publishIPNS(uploadId: string): Promise<IPNSName>;
    updateIPNSName(ipnsNameId: string, uploadId: string): Promise<IPNSName>;
    getIPNSName(ipnsNameId: string): Promise<IPNSName>;
    getIPNSNamesForUpload(uploadId: string): Promise<IPNSName[]>;
    getIPNSNamesForOrganization(organizationId: string): Promise<IPNSName[]>;
    getBucketUploadCount(bucketId: string): Promise<{
        total: number;
        successful: number;
        failed: number;
        pending: number;
    }>;
    getBucketUploads(bucketId: string, options: {
        skip: number;
        limit: number;
    }): Promise<Upload[]>;
    getUpload(uploadId: string): Promise<Upload>;
    getOrganizationUsage(organizationId: string): Promise<UsageWithLimits>;
    getTokenScope(): Promise<TokenScope>;
}

export { Bucket, BucketStateEnum, Domain, DomainTypeEnum, IPNSName, ProtocolEnum, SpheronClient, SpheronClientConfiguration, TokenScope, Upload, UploadStatusEnum, UsageWithLimits, SpheronClient as default, ipfs };
